#!/usr/bin/ruby
#require 'narray'
require 'gsl'

m = GSL_Matrix::new([0.18, 0.60, 0.57, 0.96], [0.41, 0.24, 0.99, 0.58],
		    [0.14, 0.30, 0.97, 0.66], [0.51, 0.13, 0.19, 0.85])
m.print
lu, p = m.LU_decomp
a = lu.LU_solve(p, [1, 2, 3, 4].to_gv)
p a.to_a

__END__

m = GSL_Matrix::new([5, 2, 3], [2, 1, 1], [3, 1, 9])
c = m.cholesky_decomp
x = c.cholesky_solve([2, 3, 4].to_gv)
x.print

__END__
m = GSL_Matrix::new([1, 2, 3], [6, 5, 4], [7, 8, 1])
x = m.SVD_solve([2, 3, 4].to_gv)
x.print

__END__

m = GSL_Matrix::new([1, 2, 3], [6, 5, 4], [7, 8, 1])
tau, p = m.QRPT_decomp
x = m.QRPT_Rsolve(p, [2, 3, 4].to_gv)
x.print

__END__
m = GSL_Matrix::new([1, 2, 3], [6, 5, 4], [7, 8, 1])
U, V, S = m.SV_decomp

x = gsl_linalg_SV_solve(U, V, S, [2, 3, 4].to_gv)
x.print

__END__

m = GSL_Matrix::new([1, 2, 3], [6, 5, 4], [7, 8, 1])
lu, p, signum = m.LU_decomp
x = lu.LU_solve(p, [4, 6, 2].to_gv)
p x.to_a

qr, tau = m.QR_decomp
x = qr.QR_solve(tau, [4, 6, 2].to_gv)
p x.to_a

x = [4, 6, 2].to_gv
qr.QR_svx(tau, x)
p x.to_a

__END__

tau = m.QR_decomp!
m.print
p tau.to_a

__END__

lu, p, signum = m.LU_decomp
inv = lu.LU_invert(p)
inv.print

det = lu.LU_det(signum)
p det
lndet = lu.LU_lndet
p lndet
p log(det)

__END__

inv = m.invert
inv.print
m.print

det = lu.LU_det(signum)
p det

p m.det

m = GSL_Matrix::new([0.18, 0.60, 0.57, 0.96], [0.41, 0.24, 0.99, 0.58], [0.14, 0.30, 0.97, 0.66], [0.51, 0.13, 0.19, 0.85])
m.print

b = [1, 2, 3, 4]
lu, p = m.LU_decomp
a = lu.LU_solve(p, b.to_gv)
p a.to_a
p b

__END__

m2, p = m.LU_decomp
m2.print
a = m2.LU_solve(p, [1, 2, 3, 4])
p a.to_a

__END__

m = GSL_Matrix::new(5, 5)

for i in 0...5 do
  for j in 0...5 do
    val = 0.5*(i+0.4)*(j+1.2)
    m.set(i, j, val)
  end
end

m.print

m2 = m.submatrix(1, 2, 2, 3)
m2.print

